<?php
// $Id: meetu.profile,v 1.6 2010/06/25 21:49:59 JohnKarahalis Exp $

/**
 * @file
 * Core setup tasks of the meetü Installation Profile.
 *
 * @author
 * John Karahalis (john.karahalis@rit.edu)
 */

/**
 * Username for the account that will be used by the Mission Card Processor
 * software.
 */
define('MEETU_CARD_PROCESSOR_USER', 'MC Processor');

/**
 * A short form of the username that is used by the Mission Card Processor. This
 * is primarily used as the "plus sign suffix" for the account's email address.
 */
define('MEETU_CARD_PROCESSOR_USER_SHORT', 'MCP');

/**
 * Role for the account that will be used by the Mission Card Processor.
 */
define('MEETU_CARD_PROCESSOR_ROLE', 'data provider');

/**
 * Implementation of hook_profile_details().
 *
 * Provide a name and description of this profile for the Drupal installer.
 */
function meetu_profile_details() {
  $details = array(
    'name'        => 'meetü',
    'description' => 'meetü is a Drupal-based platform for running games that encourage people to connect with one another, either for an event or as part of an organization’s ongoing effort to get to know itself.',
  );

  return $details;
}

/**
 * Implementation of hook_profile_modules().
 *
 * Return an array of modules to be enabled.
 */
function meetu_profile_modules() {
  $modules = array(

    // Core - required.
    'block',
    'filter',
    'node',
    'system',
    'user',

    // Core - optional.
    'color',
    'dblog',
    'help',
    'menu',
    'path',
    'php',
    'update',

    // CCK.
    'content',       // Dependencies: Date modules, other CCK modules.
    'text',          // Dependencies: covert_fields, nodereference, nodereferrer.
    'optionwidgets', // Dependencies: nodereference, nodereferrer.
    'computed_field',
    'content_permissions',
    'content_profile',
    'content_profile_registration',
    'fieldgroup',
    'link',
    'nodereference', // Dependencies: nodereferrer.
    'nodereferrer',
    'number',

    // Date.
    'date_api',       // Dependencies: Other Date modules.
    'date_timezone',  // Dependencies: Other Date modules.
    'date',

    // Services.
    'services', // Dependencies: Other Services modules.
    'node_service',
    'system_service',
    'user_service',
    'xmlrpc_server',

    // Views.
    'views',
    'views_bulk_operations',
    'views_customfield',
    'views_ui',
    'views_xml',

    // OPL modules.
    'covert_fields',
    'meetuset',
    'qrs_sheets',
    'qrs_sheet_bulk_render',
    'qrs_sheet_generator',
    'xor_encryption',

    // Other.
    'adminrole',
    'autoassignrole',
    'auto_nodetitle',
    'features',
    'install_profile_api',
    'mobile_codes',
    'pathauto',
    'token',

  );

  return $modules;
}

/**
 * Return an array of Features that should be enabled when this profile is
 * installed.
 *
 * @return
 *   An array of Features to enable.
 */
function meetu_features() {
  $features = array(
    'meetu_feature_core',
    'meetu_feature_p2e',
    'meetu_feature_p2p',
  );

  return $features;
}

/**
 * Implementation of hook_profile_tasks().
 *
 * Install Features and handle some tasks and settings that the Features
 * themselves do not take care of.
 */
function meetu_profile_tasks(&$task, $url) {
  if ($task == 'profile') {
    // Install Features.
    _meetu_install_features($url);
  }

  if ($task == 'meetu-setup') {
    // Enable the Install Profile API.
    install_include(meetu_profile_modules());

    // Set up everything a basic Drupal install provides.
    require_once('profiles/default/default.profile');
    default_profile_tasks($task, $url);

    // services: Don't use keys.
    variable_set('services_use_key', 0);
    
    // Disable the admin_content View, as our overridden version will be used
    // instead.
    _meetu_install_views_disable_view('admin_content');

    // Clear caches. Required for Features permissions to load correctly.
    drupal_flush_all_caches();

    // Handle more involved setup tasks.
    _meetu_setup_look();
    _meetu_setup_content_types();
    _meetu_setup_users();
    _meetu_setup_menu();
    _meetu_setup_auto_nodetitle();
    _meetu_setup_pathauto();

    /* Hide status messages generated by installed modules and drupal_execute().
       These notices are not very useful to the person installing meetü, so
       they are abstracted away. */
    _meetu_hide_status_messages();

    $task = 'profile-finished';
  }
}

/**
 * Install Features as part of a batch process.
 *
 * Gratefully borrowed and adapted from Managing News source - managingnews.com
 *
 * @param $url
 *   The <code>$url</code> provided by hook_profile_tasks() at the time this is run.
 */
function _meetu_install_features($url) {
  // Install Features as part of a batch process.
  // Gratefully borrowed and adapted from Managing News source - managingnews.com
  $modules = meetu_features();
  $files = module_rebuild_cache();
  $operations = array();
  foreach ($modules as $module) {
    $operations[] = array('_install_module_batch', array($module, $files[$module]->info['name']));
  }
  $batch = array(
    'operations' => $operations,
    'finished' => '_meetu_profile_batch_finished',
    'title' => st('Customizing Drupal for @profile platform', array('@profile' => drupal_install_profile_name())),
    'error_message' => st('The installation has encountered an error.'),
  );
  variable_set('install_task', 'profile-install-batch');
  batch_set($batch);
  batch_process($url, $url);
}

/**
 * Callback for finished Features batch install.
 *
 * Set the next installation task.
 */
function _meetu_profile_batch_finished($success, $results) {
  variable_set('install_task', 'meetu-setup');
}

/**
 * Configure settings related to the look of the site.
 */
function _meetu_setup_look() {
  // Mark Garland as enabled, which is not done by default for some reason...
  // This allows the user to customize the look of the site.
  install_enable_theme('garland');

  // Disable the "Powered by Drupal" block in Garland.
  install_disable_block('system', '0', 'garland');

  // Set the site footer to use some branding.
  variable_set('site_footer', st('meetü Game Platform – a project of the <a href="!url_opl">Open Publishing Lab @ RIT</a>.<br />&copy; 2010 The Open Publishing Lab at the Rochester Institute of Technology.', array('!url_opl' => url('http://opl.rit.edu'))));
}

/**
 * Configure settings related to content types.
 */
function _meetu_setup_content_types() {
  // Set up workflow options for content types.
  variable_set('node_options_exhibit',         array('status'));
  variable_set('node_options_achievement_p2e', array('status'));
  variable_set('node_options_mission_p2e',     array('status'));
  variable_set('node_options_achievement_p2p', array('status'));
  variable_set('node_options_mission_p2p',     array('status'));
  variable_set('node_options_profile',         array('status'));

  // Content profile settings for the "Profile" content type.
  variable_set('content_profile_use_profile', 1);
  $content_profile_options = variable_get('content_profile_profile', array());
  $content_profile_options['add_link'] = 0;
  $content_profile_options['registration_use'] = 1;
  variable_set('content_profile_profile', $content_profile_options);

  // Set up custom defaults for QR code fields.
  variable_set(QRS_FIELD_HEADER . '_' . QRS_CONTENT_TYPE,  'After the game, register online at [site-url]');
  variable_set(QRS_FIELD_FOOTER . '_' . QRS_CONTENT_TYPE,  '[site-name]');
  variable_set(QRS_FIELD_CONTENT . '_' . QRS_CONTENT_TYPE, '[field_code_shared-raw]');
  variable_set(QRS_FIELD_LABEL . '_' . QRS_CONTENT_TYPE,   '[field_code_shared-raw]');
  variable_set(QRS_FIELD_FIRST_DIFFERENT . '_' . QRS_CONTENT_TYPE, 1);

  // Set up custom defaults for the first sticker of QR-code sheets.
  $qrs_first_sticker = array(
    QRS_FIELD_LABEL => st("Keep this\nto register\nafter the\ngame!\n\nSheet Code:\n[field_code_player-raw]"),
    QRS_FIELD_FIRST_DIFFERENT => TRUE,
  );
  $qrs_first_sticker = serialize($qrs_first_sticker);
  variable_set(QRS_FIELD_FIRST_STICKER . '_' . QRS_CONTENT_TYPE, $qrs_first_sticker);
}

/**
 * Configure settings related to user management.
 */
function _meetu_setup_users() {
  // Get RIDs of some roles.
  $rid_admin  = install_get_rid('administrator');
  $rid_player = install_get_rid('player');
  $rid_data   = install_get_rid(MEETU_CARD_PROCESSOR_ROLE);

  // Grant user 1 (the "root" user) the "administrator" role.
  install_add_user_to_role(1, $rid_admin);

  /* Create the mission card processor user.
     Use a "plus sign suffix" in the user's email address so that it is not
     exactly the same as the email address for the root user. */
  $root_user       = user_load(1);
  $processor_email = str_replace('@', '+' . MEETU_CARD_PROCESSOR_USER_SHORT . '@', $root_user->mail);
  $processor_pass  = user_password(); // Defined here because user_password() is not in scope when we define constants.
  install_add_user(MEETU_CARD_PROCESSOR_USER, $processor_pass, $processor_email, array(MEETU_CARD_PROCESSOR_ROLE));

  // Set a message about the new user account.
  $processor_msg  = t('A user account was created for the meet&uuml; Mission Card Processor. The account has the following credentials:');
  $processor_msg .= '<ul><li>' . t('<strong>Username:</strong> @processor_user', array('@processor_user' => MEETU_CARD_PROCESSOR_USER))    . '</li>';
  $processor_msg .= '<li>'     . t('<strong>Password:</strong> @processor_pass', array('@processor_pass' => $processor_pass))              . '</li></ul>';
  $processor_msg .= t('To change the account credentials at any time, visit the <a href="!user_page">User Administration Page</a>.', array('!user_page' => url('admin/user/user', array('absolute' => TRUE))));
  drupal_set_message($processor_msg, 'warning');

  // autoassignrole: Enable automatic role assignment, with "player" as the automatic role.
  module_load_include('inc', 'autoassignrole', 'autoassignrole-admin');
  $form_state = array();
  $form_state['values']['auto_active'] = 1; // Enable automatic assignment.
  $form_state['values']['auto_roles']  = array($rid_player => $rid_player); // Use "player" as automatic assignment.
  drupal_execute('autoassignrole_admin_form', $form_state);

  // adminrole: Make the "administrator" role the adminrole and assign all initial permissions.
  variable_set('user_admin_role', $rid_admin);
  adminrole_update_permissions();

  // Do not require email verification for new users.
  variable_set('user_email_verification', 0);
}

/**
 * Modify the site menu.
 */
function _meetu_setup_menu() {
  // Move "Generate QR-code Sheets" and "Render QR-code Sheets" below the "Sticker sheets" menu item.
  $menu_stickersheets = install_menu_get_items('admin/content/stickersheets');
  $menu_qrs_render    = install_menu_get_items('admin/content/render-all-qrs-sheets');
  $menu_qrs_generate  = install_menu_get_items('admin/content/generate/qrs-sheets');
  install_menu_set_menu($menu_qrs_render[0]['mlid'], $menu_stickersheets[0]['mlid']);
  install_menu_set_menu($menu_qrs_generate[0]['mlid'], $menu_stickersheets[0]['mlid']);
}

/**
 * Configure the auto_nodetitle module to set up automatic node titles by
 * content type.
 */
function _meetu_setup_auto_nodetitle() {
  // Player-to-Exhibit Achievement.
  variable_set('ant_achievement_p2e', 1);
  $ant_pattern_achievement_p2e = st('Sheet ID !player-nid - Achievement for Player-to-Exhibit Mission: !mission-title', array('!player-nid' => '#[field_player-nid]', '!mission-title' => '[field_mission_p2e-title]'));
  variable_set('ant_pattern_achievement_p2e', $ant_pattern_achievement_p2e);

  // Player-to-Player Achievement.
  variable_set('ant_achievement_p2p', 1);
  $ant_pattern_achievement_p2p = st('Sheet ID !player-nid - Achievement for Player-to-Player Mission: !mission-title', array('!player-nid' => '#[field_player-nid]', '!mission-title' => '[field_mission_p2p-title]'));
  variable_set('ant_pattern_achievement_p2p', $ant_pattern_achievement_p2p);

  // Profile.
  variable_set('ant_profile', 1);
  $ant_pattern_profile = st('User Profile - !author-name', array('!author-name' => '[author-name]'));
  variable_set('ant_pattern_profile', $ant_pattern_profile);

  // QR-Code Sticker Sheet.
  variable_set('ant_qr_sticker_sheet', 1);
  $ant_pattern_qr_sticker_sheet = st('Sticker Sheet - Public ID !code-shared', array('!code-shared' => '#[field_code_shared-raw]'));
  variable_set('ant_pattern_qr_sticker_sheet', $ant_pattern_qr_sticker_sheet);
}

/**
 * Configure the pathauto module.
 */
function _meetu_setup_pathauto() {
  // Set some automatic alias defaults.
  variable_set('pathauto_node_pattern',                  'content/[title-raw]');
  variable_set('pathauto_node_exhibit_pattern',          'exhibit/[title-raw]');
  variable_set('pathauto_node_achievement_p2e_pattern',  'achievement/[field_player-nid]/p2e/[field_mission_p2e-title-raw]');
  variable_set('pathauto_node_mission_p2e_pattern',      'mission/p2e/[title-raw]');
  variable_set('pathauto_node_achievement_p2p_pattern',  'achievement/[field_player-nid]/p2p/[field_mission_p2p-title-raw]');
  variable_set('pathauto_node_mission_p2p_pattern',      'mission/p2p/[title-raw]');
  variable_set('pathauto_node_qr_sticker_sheet_pattern', 'sticker-sheet/[field_code_shared-raw]');
  variable_set('pathauto_user_pattern',                  'user/[user-raw]');
}

/**
 * Implementation of hook_form_alter().
 */
function meetu_form_alter(&$form, $form_state, $form_id) {
  // Provide some defaults on the the "Configure site" page.
  // Gratefully borrowed and adapted from Managing News source - managingnews.com
  if ($form_id == 'install_configure') {
    $default_user  = 'admin';
    $default_email = "$default_user@" . $_SERVER['HTTP_HOST'];

    $form['site_information']['site_name']['#default_value'] = drupal_install_profile_name();
    $form['site_information']['site_mail']['#default_value'] = $default_email;
    $form['admin_account']['account']['name']['#default_value'] = $default_user;
    $form['admin_account']['account']['mail']['#default_value'] = $default_email;
  }
}

/**
 * Implementation of hook_form_FORM_ID_alter().
 *
 * Make the meetu profile the default at the profile selection screen, and
 * make it the first choice to appear.
 *
 * This is in the system namespace because Drupal only invokes hooks in the
 * system namespace during install.
 *
 * Gratefully borrowed and adapted from Open Atrium source - openatrium.com
 */
function system_form_install_select_profile_form_alter(&$form, $form_state) {
  // Make meetü selected by default.
  foreach ($form['profile'] as $key => $element) {
    $form['profile'][$key]['#value'] = 'meetu';
  }

  // Make meetü appear as the first choice.
  $form['profile']['meetü']['#weight'] = -40;
}

/**
 * Remove all status messages currently in the message queue.
 */
function _meetu_hide_status_messages() {
  drupal_get_messages('status');
}

/**
 * Disables a view by name. This function is borrowed from a patch to Install
 * Profile API which hasn't been packaged into a supported version yet. When
 * this function does make it into a supported version of Install Profile API
 * (presumably version 6.x-2.2) this will no longer be needed.
 *
 * See: http://drupal.org/node/546226
 *  
 * @param $name
 *  The name of the view to disable.
 * @param $rebuild_menu
 *  The menu tree must be rebuilt after install_views_disable_view() is
 *  called, however when called many times in a row it may be sensible
 *  to only rebuild the menu tree once at the end.
 **/
function _meetu_install_views_disable_view($name, $rebuild_menu = TRUE) {
  $views_status = variable_get('views_defaults', array());
  $views_status[$name] = TRUE; // True is disabled
  variable_set('views_defaults', $views_status);
  views_invalidate_cache();

  if ($rebuild_menu) {
    menu_rebuild();
  } 
} 

